//
//  BMKPOICitySearchViewController.m
//  BMKPlaceSearchDemo
//
//  Created by Baidu on 2020/5/24.
//  Copyright © 2020 Baidu. All rights reserved.
//

#import "BMKPOICitySearchViewController.h"
#import "UIImage+BMKImage.h"
#import "BMKPoiCell.h"

@interface BMKPOICitySearchViewController ()<BMKMapViewDelegate, UISearchBarDelegate, BMKPoiSearchDelegate, UITableViewDataSource, UITableViewDelegate>

/// 当前界面的mapView
@property (nonatomic, strong) BMKMapView *mapView;

/// 地点提示tableView
@property (nonatomic, strong) UITableView *tableView;

/// poilist
@property (nonatomic, copy) NSArray<BMKPoiInfo *> *poiList;

/// 自定义检索View
@property (nonatomic, strong) UIView *searchView;

/// 当前界面选择的poi展示label
@property (nonatomic, strong) UILabel *addressLabel;

@end

@implementation BMKPOICitySearchViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.navigationController.navigationBar.translucent = NO;
    self.view.backgroundColor = [UIColor whiteColor];
    self.title = @"地点检索";
    [self.view addSubview:self.mapView];
    [self.view addSubview:self.tableView];
    [self.view addSubview:self.addressLabel];
    [self.view addSubview:self.searchView];
    // 监听键盘弹出
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardDidShow:) name:UIKeyboardDidShowNotification object:nil];
    // 监听键盘退出
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillHide:) name:UIKeyboardWillHideNotification object:nil];

}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:NO];
    // 当mapView即将被显示的时候调用，恢复之前存储的mapView状态
    [_mapView viewWillAppear];
    [self.searchView  endEditing:YES];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:NO];
    // 当mapView即将被隐藏的时候调用，存储当前mapView的状态
    [_mapView viewWillDisappear];
    [self.searchView  endEditing:YES];
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)keyboardDidShow:(NSNotification *)notification {
    self.tableView.hidden = NO;
}
- (void)keyboardWillHide:(NSNotification *)notification {
    self.addressLabel.hidden = YES;
}
// 根据输入发送检索
- (void)searchPoiInfoWithKey:(NSString *)key {
    if (key.length == 0) return;
    BMKPOICitySearchOption *option = [BMKPOICitySearchOption new];
    option.keyword = key;
    option.city = @"北京市";
    option.isCityLimit = YES;
    
    BMKPoiSearch *search = [BMKPoiSearch new];
    search.delegate = self;
    [search poiSearchInCity:option];

}

#pragma mark - BMKPoiSearchDelegate

/// 返回POI搜索结果
/// @param searcher 搜索对象
/// @param poiResult 搜索结果列表
/// @param errorCode BMKSearchErrorCode
- (void)onGetPoiResult:(BMKPoiSearch *)searcher result:(BMKPOISearchResult *)poiResult errorCode:(BMKSearchErrorCode)errorCode {
    if (errorCode ==  BMK_SEARCH_NO_ERROR) {
        self.poiList = [poiResult.poiInfoList copy];
        __weak typeof(self) weakSelf = self;
        dispatch_async(dispatch_get_main_queue(), ^{
            [weakSelf.tableView reloadData];
        });
    }
}
#pragma mark - BMKMapViewDelegate

/// 根据anntation生成对应的annotationView
/// @param mapView 地图View
/// @param annotation 指定的标注
- (BMKAnnotationView *)mapView:(BMKMapView *)mapView viewForAnnotation:(id<BMKAnnotation>)annotation {
    if ([annotation isKindOfClass:[BMKPointAnnotation class]]) {
        BMKPinAnnotationView *annotationView = (BMKPinAnnotationView *)[mapView dequeueReusableAnnotationViewWithIdentifier:@"com.Baidu.BMKPointAnnotation"];
        if (!annotationView) {
            annotationView = [[BMKPinAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:@"com.Baidu.BMKPointAnnotation"];
            annotationView.image = [UIImage imageNamed:@"icon_marker"];
            annotationView.canShowCallout = YES;
            annotationView.selected = YES;
            annotationView.pinColor = BMKPinAnnotationColorRed;
        }
        return annotationView;
    }
    return nil;
}
#pragma mark - UITableViewDataSource

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.poiList.count;
}
- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 70;
}
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *tipCellIdentifier = @"poiList.cell";
    BMKPoiCell *cell = [tableView dequeueReusableCellWithIdentifier:tipCellIdentifier];
    if (cell == nil) {
        cell = [[BMKPoiCell alloc] initWithStyle:UITableViewCellStyleSubtitle
                                      reuseIdentifier:tipCellIdentifier];
    }
    BMKPoiInfo *info = self.poiList[indexPath.row];
    [cell configTitle:info.name subTitle:info.address];
    return cell;
}

#pragma mark - UITableViewDelegate
// 选中cell
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [self.mapView removeAnnotations:self.mapView.annotations];
    [self.searchView  endEditing:YES];
    BMKPoiInfo *info = self.poiList[indexPath.row];
    if (info.UID != nil && CLLocationCoordinate2DIsValid(info.pt)) {
        BMKPointAnnotation *annotaion = [[BMKPointAnnotation alloc] init];
        annotaion.coordinate = info.pt;
        // 添加标注
        [_mapView addAnnotation:annotaion];
        // 设置地图中心点
        [_mapView setCenterCoordinate:info.pt];
        NSString *name = info.name ? info.name : @"";
        NSString *address = info.address ? info.address : @"";
        NSMutableAttributedString *poi = [[NSMutableAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n%@", name, address]];
        NSRange nRange = NSMakeRange(0, [name length]);
        NSRange aRange = NSMakeRange([name length], [address length] + 1);
        [poi addAttribute:NSFontAttributeName value:[UIFont fontWithName:@"PingFangSC-Semibold" size:24] range:nRange];
        [poi addAttribute:NSFontAttributeName value:[UIFont fontWithName:@"PingFangSC-Regular" size:16] range:aRange];
        // 底部label展示poi信息
        _addressLabel.attributedText = poi;
        _addressLabel.hidden = NO;
    }
    self.tableView.hidden = YES;
    
}
 
#pragma mark - UISearchBarDelegate

// 输入回调
- (void)searchBar:(UISearchBar *)searchBar textDidChange:(NSString *)searchText {
     [self searchPoiInfoWithKey:searchText];
}
// 确认检索
- (void)searchBarSearchButtonClicked:(UISearchBar *)searchBar {
     [self searchPoiInfoWithKey:searchBar.text];
}

#pragma mark - UI
- (BMKMapView *)mapView {
    if (!_mapView) {
        _mapView = [[BMKMapView alloc] initWithFrame:CGRectMake(0, 0, KScreenWidth, CGRectGetHeight(self.view.frame) - kViewTopHeight)];
        _mapView.delegate = self;
        _mapView.zoomLevel = 16;
        _mapView.trafficEnabled = YES;
    }
    return _mapView;
}

- (UITableView *)tableView {
    if (!_tableView) {
        _tableView = [[UITableView alloc] initWithFrame:CGRectMake(0, CGRectGetMaxY(self.searchView.frame), CGRectGetWidth(self.view.frame), CGRectGetHeight(self.view.frame) - CGRectGetMaxY(self.searchView.frame)) style:UITableViewStylePlain];
        _tableView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
        _tableView.delegate = self;
        _tableView.dataSource = self;
        _tableView.hidden = YES;
        _tableView.keyboardDismissMode = UIScrollViewKeyboardDismissModeOnDrag;
    }
    return _tableView;
}
- (UIView *)searchView {
    if (!_searchView) {
        _searchView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, KScreenWidth, 80)];
        _searchView.backgroundColor = [UIColor colorWithRed:68.f / 255.f green:70.f / 255.f blue:86.f / 255.f alpha:1.f];
        UIView *containerView = [[UIView alloc] initWithFrame:CGRectMake(10, 20, KScreenWidth - 10 * 2, 40)];
        containerView.backgroundColor = [UIColor colorWithRed:41.f / 255.f green:43.f / 255.f blue:59.f / 255.f alpha:1.f];
        containerView.layer.cornerRadius = 5;
        [_searchView addSubview:containerView];
        UILabel *label = [[UILabel alloc] initWithFrame:CGRectMake(20, 0, 120, 40)];
        label.font = [UIFont systemFontOfSize:16];
        label.textColor = [UIColor colorWithRed:111.f / 255.f green:112.f / 255.f blue:123.f / 255.f alpha:1.f];
        NSMutableAttributedString *tipStr = [[NSMutableAttributedString alloc] initWithString:@"在 北京 市内找 "];
        NSRange tipRangre = NSMakeRange(2, 2);
        [tipStr addAttribute:NSForegroundColorAttributeName value:[UIColor whiteColor] range:tipRangre];
        [tipStr addAttribute:NSFontAttributeName value:[UIFont systemFontOfSize:17] range:tipRangre];
        label.attributedText = tipStr;
        [containerView addSubview:label];
        UISearchBar *searchBar = [[UISearchBar alloc] initWithFrame:CGRectMake(CGRectGetMaxX(label.frame), CGRectGetMinY(label.frame), CGRectGetWidth(containerView.frame) - CGRectGetWidth(label.frame) - 20, 40)];
        searchBar.delegate = self;
        searchBar.searchBarStyle = UISearchBarStyleMinimal;
        // 取消搜索图片
        [searchBar setImage:[UIImage bmk_getImageWithColor:[UIColor clearColor] height:40] forSearchBarIcon:UISearchBarIconSearch state:UIControlStateNormal];
        searchBar.backgroundImage = [UIImage bmk_getImageWithColor:[UIColor clearColor] height:40];
        [searchBar setSearchFieldBackgroundImage:[UIImage bmk_getImageWithColor:[UIColor clearColor] height:40] forState:UIControlStateNormal];
        [searchBar setTintColor:[UIColor whiteColor]];
        [searchBar setBarTintColor:[UIColor whiteColor]];
        searchBar.placeholder = @"请输入关键字";
         // 文本颜色
        if (@available(iOS 9.0, *)) {
            [UITextField appearanceWhenContainedInInstancesOfClasses:@[[UISearchBar class]]].textColor = [UIColor whiteColor];
        } else {
            [[UITextField appearanceWhenContainedIn:[UISearchBar class], nil] setTextColor:[UIColor whiteColor]];
        }
     
        [containerView addSubview:searchBar];
    }
    return _searchView;
}
- (UILabel *)addressLabel {
    if (!_addressLabel) {
        _addressLabel = [[UILabel alloc] initWithFrame:CGRectMake(10, CGRectGetMaxY(_mapView.frame) - 80 - 15, KScreenWidth - 10 * 2, 80)];
        _addressLabel.textAlignment = NSTextAlignmentCenter;
        _addressLabel.numberOfLines = 0;
        _addressLabel.backgroundColor = [UIColor whiteColor];
        _addressLabel.textColor = [UIColor colorWithRed:51.f / 255.f green:51.f / 255.f blue:51.f / 255.f alpha:1.f];
        _addressLabel.layer.cornerRadius = 5;
        _addressLabel.layer.masksToBounds = YES;
        _addressLabel.hidden = YES;
    }
    return _addressLabel;
}
@end
